# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long, protected-access, too-few-public-methods
from knack.log import get_logger

from azure.cli.core.aaz import has_value, register_command, AAZResourceIdArgFormat, AAZListArg, AAZResourceIdArg
from azure.cli.core.aaz.utils import assign_aaz_list_arg
from ..aaz.latest.network.dns.record_set import Update as _RecordSetUpdate, Show as _RecordSetShow, \
    ListByType as _RecordSetListByType, Delete as _RecordSetDelete, Create as _RecordSetCreate
from ..aaz.latest.network.dns import ListReferences as _DNSListReferences

logger = get_logger(__name__)


@register_command("network dns list-references")
class DNSListReferences(_DNSListReferences):
    """ Returns the DNS records specified by the referencing targetResourceIds.

    :example: Returns the DNS records specified by the referencing targetResourceIds.
        az network dns list-references --parameters "/subscriptions/**921/resourceGroups/MyResourceGroup/providers/Microsoft.Network/trafficManagerProfiles/MyTrafficManager"
    """
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.target_resources._registered = False

        args_schema.parameters = AAZListArg(
            options=["--parameters"],
            help="A space-separated list of resource IDs for which referencing dns records need to be queried.",
        )

        parameters = args_schema.parameters
        parameters.Element = AAZResourceIdArg()

        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.target_resources = assign_aaz_list_arg(
            args.target_resources,
            args.parameters,
            element_transformer=lambda _, x: {"id": x})


# region RecordSetUpdate
class RecordSetUpdate(_RecordSetUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.target_resource._fmt = AAZResourceIdArgFormat()

        args_schema.record_type._required = False
        args_schema.record_type._registered = False

        args_schema.ttl._registered = False
        args_schema.a_records._registered = False
        args_schema.aaaa_records._registered = False
        args_schema.caa_records._registered = False
        args_schema.cname_record._registered = False
        args_schema.ds_records._registered = False
        args_schema.mx_records._registered = False
        args_schema.naptr_records._registered = False
        args_schema.ns_records._registered = False
        args_schema.ptr_records._registered = False
        args_schema.soa_record._registered = False
        args_schema.srv_records._registered = False
        args_schema.tlsa_records._registered = False
        args_schema.txt_records._registered = False

        args_schema.naptr_records._registered = False
        return args_schema

    def post_instance_update(self, instance):
        if not has_value(instance.properties.target_resource.id):
            instance.properties.target_resource = None


@register_command("network dns record-set a update")
class RecordSetAUpdate(RecordSetUpdate):
    """ Update an A record set.

    :example: Update an A record set.
        az network dns record-set a update -g MyResourceGroup -n MyRecordSet -z www.mysite.com --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "A"


@register_command("network dns record-set aaaa update")
class RecordSetAAAAUpdate(RecordSetUpdate):
    """ Update an AAAA record set.

    :example: Update an AAAA record set.
        az network dns record-set aaaa update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "AAAA"


@register_command("network dns record-set ds update")
class RecordSetDSUpdate(RecordSetUpdate):
    """ Update an DS record set.

    :example: Update an DS record set.
        az network dns record-set ds update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "DS"


@register_command("network dns record-set mx update")
class RecordSetMXUpdate(RecordSetUpdate):
    """ Update an MX record set.

    :example: Update an MX record set.
        az network dns record-set mx update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "MX"


@register_command("network dns record-set naptr update")
class RecordSetNAPTRUpdate(RecordSetUpdate):
    """ Update an NAPTR record set.

    :example: Update an NAPTR record set.
        az network dns record-set naptr update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NAPTR"


@register_command("network dns record-set ns update")
class RecordSetNSUpdate(RecordSetUpdate):
    """ Update an NS record set.

    :example: Update an NS record set.
        az network dns record-set ns update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NS"


@register_command("network dns record-set ptr update")
class RecordSetPTRUpdate(RecordSetUpdate):
    """ Update a PTR record set.

    :example: Update a PTR record set.
        az network dns record-set ptr update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "PTR"


@register_command("network dns record-set srv update")
class RecordSetSRVUpdate(RecordSetUpdate):
    """ Update an SRV record set.

    :example: Update an SRV record set.
        az network dns record-set srv update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SRV"


@register_command("network dns record-set tlsa update")
class RecordSetTLSAUpdate(RecordSetUpdate):
    """ Update a TLSA record set.

    :example: Update a TLSA record set.
        az network dns record-set tlsa update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TLSA"


@register_command("network dns record-set txt update")
class RecordSetTXTUpdate(RecordSetUpdate):
    """ Update a TXT record set.

    :example: Update a TXT record set.
        az network dns record-set txt update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TXT"


@register_command("network dns record-set caa update")
class RecordSetCAAUpdate(RecordSetUpdate):
    """ Update a CAA record set.

    :example: Update a CAA record set.
        az network dns record-set caa update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CAA"


@register_command("network dns record-set cname update")
class RecordSetCNAMEUpdate(RecordSetUpdate):
    """ Update a CNAME record set.

    :example: Update a CNAME record set.
        az network dns record-set cname update -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CNAME"
# endregion RecordSetUpdate


# region RecordSetShow
class RecordSetShow(_RecordSetShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.record_type._required = False
        args_schema.record_type._registered = False

        return args_schema


@register_command("network dns record-set a show")
class RecordSetAShow(RecordSetShow):
    """ Get an A record set.

    :example: Get an A record set.
        az network dns record-set a show -g MyResourceGroup -n MyRecordSet -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "A"


@register_command("network dns record-set aaaa show")
class RecordSetAAAAShow(RecordSetShow):
    """ Get an AAAA record set.

    :example: Get an AAAA record set.
        az network dns record-set aaaa show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "AAAA"


@register_command("network dns record-set ds show")
class RecordSetDSShow(RecordSetShow):
    """ Get an DS record set.

    :example: Get an DS record set.
        az network dns record-set ds show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "DS"


@register_command("network dns record-set mx show")
class RecordSetMXShow(RecordSetShow):
    """ Get an MX record set.

    :example: Get an MX record set.
        az network dns record-set mx show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "MX"


@register_command("network dns record-set naptr show")
class RecordSetNAPTRShow(RecordSetShow):
    """ Get an NAPTR record set.

    :example: Get an NAPTR record set.
        az network dns record-set naptr show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NAPTR"


@register_command("network dns record-set ns show")
class RecordSetNSShow(RecordSetShow):
    """ Get an NS record set.

    :example: Get an NS record set.
        az network dns record-set ns show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NS"


@register_command("network dns record-set ptr show")
class RecordSetPTRShow(RecordSetShow):
    """ Get a PTR record set.

    :example: Get a PTR record set.
        az network dns record-set ptr show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "PTR"


@register_command("network dns record-set srv show")
class RecordSetSRVShow(RecordSetShow):
    """ Get an SRV record set.

    :example: Get an SRV record set.
        az network dns record-set srv show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SRV"


@register_command("network dns record-set tlsa show")
class RecordSetTLSAShow(RecordSetShow):
    """ Get a TLSA record set.

    :example: Get a TLSA record set.
        az network dns record-set tlsa show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TLSA"


@register_command("network dns record-set txt show")
class RecordSetTXTShow(RecordSetShow):
    """ Get a TXT record set.

    :example: Get a TXT record set.
        az network dns record-set txt show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TXT"


@register_command("network dns record-set caa show")
class RecordSetCAAShow(RecordSetShow):
    """ Get a CAA record set.

    :example: Get a CAA record set.
        az network dns record-set caa show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CAA"


@register_command("network dns record-set cname show")
class RecordSetCNAMEShow(RecordSetShow):
    """ Get a CNAME record set.

    :example: Get a CNAME record set.
        az network dns record-set cname show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CNAME"


@register_command("network dns record-set soa show")
class RecordSetSOAShow(RecordSetShow):
    """ Get a SOA record set.

    :example: Get a SOA record set.
        az network dns record-set soa show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.name._required = False
        args_schema.name._registered = False

        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SOA"
        args.name = "@"
# endregion RecordSetShow


# region RecordSetList
class RecordSetList(_RecordSetListByType):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.record_type._required = False
        args_schema.record_type._registered = False

        return args_schema


@register_command("network dns record-set a list")
class RecordSetAList(RecordSetList):
    """ List A record sets in a zone.

    :example: List A record sets in a zone.
        az network dns record-set a list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "A"


@register_command("network dns record-set aaaa list")
class RecordSetAAAAList(RecordSetList):
    """ List AAAA record sets in a zone.

    :example: List AAAA record sets in a zone.
        az network dns record-set aaaa list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "AAAA"


@register_command("network dns record-set ds list")
class RecordSetDSList(RecordSetList):
    """ List DS record sets in a zone.

    :example: List DS record sets in a zone.
        az network dns record-set ds list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "DS"


@register_command("network dns record-set mx list")
class RecordSetMXList(RecordSetList):
    """ List MX record sets in a zone.

    :example: List MX record sets in a zone.
        az network dns record-set mx list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "MX"


@register_command("network dns record-set naptr list")
class RecordSetNAPTRList(RecordSetList):
    """ List NAPTR record sets in a zone.

    :example: List NAPTR record sets in a zone.
        az network dns record-set naptr list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NAPTR"


@register_command("network dns record-set ns list")
class RecordSetNSList(RecordSetList):
    """ List NS record sets in a zone.

    :example: List NS record sets in a zone.
        az network dns record-set ns list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NS"


@register_command("network dns record-set ptr list")
class RecordSetPTRList(RecordSetList):
    """ List PTR record sets in a zone.

    :example: List PTR record sets in a zone.
        az network dns record-set ptr list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "PTR"


@register_command("network dns record-set srv list")
class RecordSetSRVList(RecordSetList):
    """ List SRV record sets in a zone.

    :example: List SRV record sets in a zone.
        az network dns record-set srv list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SRV"


@register_command("network dns record-set tlsa list")
class RecordSetTLSAList(RecordSetList):
    """ List TLSA record sets in a zone.

    :example: List TLSA record sets in a zone.
        az network dns record-set tlsa list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TLSA"


@register_command("network dns record-set txt list")
class RecordSetTXTList(RecordSetList):
    """ List TXT record sets in a zone.

    :example: List TXT record sets in a zone.
        az network dns record-set txt list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TXT"


@register_command("network dns record-set caa list")
class RecordSetCAAList(RecordSetList):
    """ List CAA record sets in a zone.

    :example: List CAA record sets in a zone.
        az network dns record-set caa list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CAA"


@register_command("network dns record-set cname list")
class RecordSetCNAMEList(RecordSetList):
    """ List CNAME record sets in a zone.

    :example: List CNAME record sets in a zone.
        az network dns record-set cname list -g MyResourceGroup -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CNAME"
# endregion RecordSetList


# region RecordSetDelete
class RecordSetDelete(_RecordSetDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.record_type._required = False
        args_schema.record_type._registered = False

        return args_schema


@register_command("network dns record-set a delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetADelete(RecordSetDelete):
    """ Delete an A record set.

    :example: Delete an A record set.
        az network dns record-set a delete -g MyResourceGroup -n MyRecordSet -z www.mysite.com
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "A"


@register_command("network dns record-set aaaa delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetAAAADelete(RecordSetDelete):
    """ Delete an AAAA record set.

    :example: Delete an AAAA record set.
        az network dns record-set aaaa delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "AAAA"


@register_command("network dns record-set ds delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetDSDelete(RecordSetDelete):
    """ Delete an DS record set.

    :example: Delete an DS record set.
        az network dns record-set ds delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "DS"


@register_command("network dns record-set mx delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetMXDelete(RecordSetDelete):
    """ Delete an MX record set.

    :example: Delete an MX record set.
        az network dns record-set mx delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "MX"


@register_command("network dns record-set naptr delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetNAPTRDelete(RecordSetDelete):
    """ Delete an NAPTR record set.

    :example: Delete an NAPTR record set.
        az network dns record-set naptr delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NAPTR"


@register_command("network dns record-set ns delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetNSDelete(RecordSetDelete):
    """ Delete an NS record set.

    :example: Delete an NS record set.
        az network dns record-set ns delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NS"


@register_command("network dns record-set ptr delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetPTRDelete(RecordSetDelete):
    """ Delete a PTR record set.

    :example: Delete a PTR record set.
        az network dns record-set ptr delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "PTR"


@register_command("network dns record-set srv delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetSRVDelete(RecordSetDelete):
    """ Delete an SRV record set.

    :example: Delete an SRV record set.
        az network dns record-set srv delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SRV"


@register_command("network dns record-set tlsa delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetTLSADelete(RecordSetDelete):
    """ Delete a TLSA record set.

    :example: Delete a TLSA record set.
        az network dns record-set tlsa delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TLSA"


@register_command("network dns record-set txt delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetTXTDelete(RecordSetDelete):
    """ Delete a TXT record set.

    :example: Delete a TXT record set.
        az network dns record-set txt delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TXT"


@register_command("network dns record-set caa delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetCAADelete(RecordSetDelete):
    """ Delete a CAA record set.

    :example: Delete a CAA record set.
        az network dns record-set caa delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CAA"


@register_command("network dns record-set cname delete", confirmation="Are you sure you want to perform this operation?")
class RecordSetCNAMEDelete(RecordSetDelete):
    """ Delete a CNAME record set.

    :example: Delete a CNAME record set.
        az network dns record-set cname delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CNAME"
# endregion RecordSetDelete


# region RecordSetCreate
class RecordSetCreate(_RecordSetCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.target_resource._fmt = AAZResourceIdArgFormat()

        args_schema.record_type._required = False
        args_schema.record_type._registered = False

        args_schema.a_records._registered = False
        args_schema.aaaa_records._registered = False
        args_schema.caa_records._registered = False
        args_schema.cname_record._registered = False
        args_schema.ds_records._registered = False
        args_schema.mx_records._registered = False
        args_schema.ns_records._registered = False
        args_schema.ptr_records._registered = False
        args_schema.soa_record._registered = False
        args_schema.srv_records._registered = False
        args_schema.tlsa_records._registered = False
        args_schema.txt_records._registered = False

        args_schema.naptr_records._registered = False
        return args_schema


@register_command("network dns record-set a create")
class RecordSetACreate(RecordSetCreate):
    """ Create an A record set.

    :example: Create an A record set.
        az network dns record-set a create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "A"


@register_command("network dns record-set aaaa create")
class RecordSetAAAACreate(RecordSetCreate):
    """ Create an AAAA record set.

    :example: Create an AAAA record set.
        az network dns record-set aaaa create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "AAAA"


@register_command("network dns record-set ds create")
class RecordSetDSCreate(RecordSetCreate):
    """ Create an DS record set.

    :example: Create an DS record set.
        az network dns record-set ds create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "DS"


@register_command("network dns record-set mx create")
class RecordSetMXCreate(RecordSetCreate):
    """ Create an MX record set.

    :example: Create an MX record set.
        az network dns record-set mx create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "MX"


@register_command("network dns record-set naptr create")
class RecordSetNAPTRCreate(RecordSetCreate):
    """ Create an NAPTR record set.

    :example: Create an NAPTR record set.
        az network dns record-set naptr create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NAPTR"


@register_command("network dns record-set ns create")
class RecordSetNSCreate(RecordSetCreate):
    """ Create an NS record set.

    :example: Create an NS record set.
        az network dns record-set ns create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "NS"


@register_command("network dns record-set ptr create")
class RecordSetPTRCreate(RecordSetCreate):
    """ Create a PTR record set.

    :example: Create a PTR record set.
        az network dns record-set ptr create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "PTR"


@register_command("network dns record-set srv create")
class RecordSetSRVCreate(RecordSetCreate):
    """ Create an SRV record set.

    :example: Create an SRV record set.
        az network dns record-set srv create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SRV"


@register_command("network dns record-set tlsa create")
class RecordSetTLSACreate(RecordSetCreate):
    """ Create a TLSA record set.

    :example: Create a TLSA record set.
        az network dns record-set tlsa create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TLSA"


@register_command("network dns record-set txt create")
class RecordSetTXTCreate(RecordSetCreate):
    """ Create a TXT record set.

    :example: Create a TXT record set.
        az network dns record-set txt create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "TXT"


@register_command("network dns record-set caa create")
class RecordSetCAACreate(RecordSetCreate):
    """ Create a CAA record set.

    :example: Create a CAA record set.
        az network dns record-set caa create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CAA"


@register_command("network dns record-set cname create")
class RecordSetCNAMECreate(RecordSetCreate):
    """ Create a CNAME record set.

    :example: Create a CNAME record set.
        az network dns record-set cname create -g MyResourceGroup -z www.mysite.com -n MyRecordSet --metadata owner=WebTeam
    """
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "CNAME"


class RecordSetSOACreate(RecordSetCreate):
    def pre_operations(self):
        args = self.ctx.args
        args.record_type = "SOA"
# endregion RecordSetCreate



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
import time
from azure.cli.testsdk import ScenarioTest
from azure.cli.testsdk.decorators import serial_test

LOCATION = "northeurope"
VNET_LOCATION = "northeurope"
VNET_NAME = "vnetnortheurope-anf"
RESOURCE_GROUP = "sdk-net-test-qa2"
PPG = "/subscriptions/69a75bda-882e-44d5-8431-63421204132a/resourceGroups/sdk-net-test-qa2/providers/Microsoft.Compute/proximityPlacementGroups/sdk_test_northeurope_ppg"


class AzureNetAppFilesVolumeGroupServiceScenarioTest(ScenarioTest):
    def prepare_for_volume_group_creation(self):
        # Create account and pool
        self.kwargs.update({
            'loc': LOCATION,
            'rg': RESOURCE_GROUP,
            'acc': self.create_random_name(prefix='cli-acc-', length=24),
            'pool': self.create_random_name(prefix='cli-pool-', length=24),
            'service_level': 'Premium',
            'pool_size': 20,
            'qos': 'Manual'
        })
        self.cmd("az netappfiles account create -g {rg} -a {acc} -l {loc}")
        self.cmd("az netappfiles pool create -g {rg} -a {acc} -p {pool} -l {loc} --service-level {service_level} "
                 "--size {pool_size} --qos-type {qos}")
#    @unittest.skip('(drp failure) DRP stamp pinning failing on the environment, no way to test until fixed')
    def test_create_get_list_delete_volume_group(self):
        # Create Volume Group with defaults
        self.prepare_for_volume_group_creation()
        self.kwargs.update({
            'vg_name': 'cli-test-volume-group-001',
            'vnet': VNET_NAME,
            'application_identifier': "sh1",
            'application_type': "SAP-HANA",
            'ppg': PPG,
            'zones': 1,
            'sap_sid': "CLI",
            'gpr': "'key1=value1' 'key2=value2'"
        })
        self.cmd("az netappfiles volume-group create -g {rg} -a {acc} -p {pool} --volume-group-name {vg_name} "
                 "--vnet {vnet} --zones {zones} --application-identifier {application_identifier} -l {loc} --application-type {application_type} --gp-rules {gpr}",
                 checks=[
                     self.check('name', '{acc}/{vg_name}'),
                     self.check('groupMetaData.volumesCount', 5),
                     self.check('groupMetaData.applicationIdentifier', '{application_identifier}'),
                     self.check('length(volumes)', 5)
                 ])

        # Get Volume Group
        self.cmd("az netappfiles volume-group show -g {rg} -a {acc} --volume-group-name {vg_name}",
                 checks=[
                     self.check('name', '{acc}/{vg_name}'),
                     self.check('groupMetaData.volumesCount', 5),
                     self.check('groupMetaData.applicationIdentifier', '{application_identifier}'),
                     self.check('length(volumes)', 5)
                 ])

        # List Volume Groups
        self.cmd("az netappfiles volume-group list -g {rg} -a {acc}",
                 checks=[
                     self.check('length(@)', 1)
                 ])

        # Cleanup
        self.kwargs.update({
            'vol_1': 'sh1-data-mnt00001',
            'vol_2': 'sh1-log-mnt00001',
            'vol_3': 'sh1-shared',
            'vol_4': 'sh1-data-backup',
            'vol_5': 'sh1-log-backup'
        })
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_1} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_2} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_3} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_4} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_5} --yes")

        if (self.is_live or self.in_recording):
            time.sleep(20)
        # Delete Volume Group
        self.cmd("az netappfiles volume-group delete -g {rg} -a {acc} --volume-group-name {vg_name} -y --yes")

        # Delete pool and account
        self.cmd("az netappfiles pool delete -g {rg} -a {acc} -p {pool} -y")
        if (self.is_live or self.in_recording):
            time.sleep(20)
        self.cmd("az netappfiles account delete -g {rg} -a {acc} -y")

    #@unittest.skip('(drp failure) DRP stamp pinning failing on the environment, no way to test until fixed')
    def test_oracle_volume_groups(self):
        # Create Volume Group with minimum size
        self.prepare_for_volume_group_creation()
        self.kwargs.update({
            'vg_name': 'cli-test-volume-group-001',
            'vnet': VNET_NAME,
            'ppg': PPG,
            'zones': 1,
            'application_identifier': "or1",
            'application_type': "ORACLE",
            'add_snap_cap': 0,
            'size': 100,
            'throughput': 64
        })
        self.cmd("az netappfiles volume-group create -g {rg} -a {acc} -p {pool} --group-name {vg_name} --vnet {vnet} "
                 "--zones {zones} --application-identifier {application_identifier} --application-type {application_type} -l {loc} --add-snapshot-capacity {add_snap_cap} "
                 "--database-size {size} --database-throughput 120 --number-of-volumes 2",
                 checks=[
                     self.check('name', '{acc}/{vg_name}'),
                     self.check('groupMetaData.volumesCount', 6),
                     self.check('groupMetaData.applicationIdentifier', '{application_identifier}'),
                     self.check('length(volumes)', 6)
                 ])


        # Cleanup
        self.kwargs.update({
            'vol_1': 'or1-ora-data1',
            'vol_2': 'or1-ora-data2',
            'vol_3': 'or1-ora-log',
            'vol_4': 'or1-ora-binary',
            'vol_5': 'or1-ora-backup',
            'vol_6': 'or1-ora-log-mirror'
        })
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_1} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_2} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_3} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_4} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_5} --yes")
        self.cmd("az netappfiles volume delete -g {rg} -a {acc} -p {pool} -v {vol_6} --yes")

        if (self.is_live or self.in_recording):
            time.sleep(20)
        # Delete Volume Group
        self.cmd("az netappfiles volume-group delete -g {rg} -a {acc} --group-name {vg_name} -y")
        if (self.is_live or self.in_recording):
            time.sleep(20)
        # Delete pool and account
        self.cmd("az netappfiles pool delete -g {rg} -a {acc} -p {pool} -y")
        if (self.is_live or self.in_recording):
            time.sleep(20)
        self.cmd("az netappfiles account delete -g {rg} -a {acc} -y")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import json
from knack.util import CLIError
from azure.cli.core.azclierror import (
    UnknownError
)
from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.profiles import ResourceType
from azure.cli.command_modules.acs.azuremonitormetrics.constants import (
    RP_API,
    AKS_CLUSTER_API
)


def sanitize_resource_id(resource_id):
    resource_id = resource_id.strip()
    if not resource_id.startswith("/"):
        resource_id = "/" + resource_id
    if resource_id.endswith("/"):
        resource_id = resource_id.rstrip("/")
    return resource_id.lower()


def post_request(cmd, rp_name):
    resource_client = get_mgmt_service_client(cmd.cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)
    try:
        resource_client.providers.register(rp_name)
    except Exception as e:
        raise CLIError(e)


def register_rps(cmd, subscription_id, rp_namespaces, user_agent):
    from azure.cli.core.util import send_raw_request
    try:
        headers = ['User-Agent=' + user_agent]
        armendpoint = cmd.cli_ctx.cloud.endpoints.resource_manager
        customUrl = "{0}/subscriptions/{1}/providers?api-version={2}&$select=namespace,registrationstate".format(
            armendpoint,
            subscription_id,
            RP_API
        )
        r = send_raw_request(cmd.cli_ctx, "GET", customUrl, headers=headers)
        json_response = json.loads(r.text)
        values_array = json_response["value"]

        for value in values_array:
            namespace = value["namespace"].lower()
            if namespace in rp_namespaces and value["registrationState"].lower() == "registered":
                rp_namespaces[namespace] = True

        for namespace, registered in rp_namespaces.items():
            if not registered:
                headers = ['User-Agent=azuremonitormetrics.register_{}_rp'.format(namespace.split('.')[1].lower())]
                post_request(cmd, namespace)
    except CLIError as e:
        raise CLIError(e)


def rp_registrations(cmd, cluster_subscription_id, raw_parameters):
    from azure.mgmt.core.tools import parse_resource_id
    cluster_rp_namespaces = {
        "microsoft.insights": False,
        "microsoft.alertsmanagement": False
    }
    cluster_user_agent = 'azuremonitormetrics.get_cluster_sub_rp_list'
    register_rps(cmd, cluster_subscription_id, cluster_rp_namespaces, cluster_user_agent)

    subscription_id = cluster_subscription_id
    azure_monitor_workspace_resource_id = raw_parameters.get("azure_monitor_workspace_resource_id")
    if azure_monitor_workspace_resource_id and azure_monitor_workspace_resource_id != "":
        parsed_dict = parse_resource_id(azure_monitor_workspace_resource_id)
        subscription_id = parsed_dict["subscription"]
    monitor_workspace_rp_namespaces = {
        "microsoft.insights": False,
        "microsoft.alertsmanagement": False,
        "microsoft.monitor": False
    }
    monitor_workspace_user_agent = 'azuremonitormetrics.get_monitor_workspace_rp_list'
    register_rps(cmd, subscription_id, monitor_workspace_rp_namespaces, monitor_workspace_user_agent)

    grafana_workspace_resource_id = raw_parameters.get("grafana_workspace_resource_id")
    if grafana_workspace_resource_id and grafana_workspace_resource_id != "":
        grafana_sub_id = grafana_workspace_resource_id.split("/")[2]
        grafana_rp_namespaces = {
            "microsoft.dashboard": False
        }
        grafana_user_agent = 'azuremonitormetrics.get_grafana_sub_rp_list'
        register_rps(cmd, grafana_sub_id, grafana_rp_namespaces, grafana_user_agent)


# pylint: disable=line-too-long
def check_azuremonitormetrics_profile(cmd, cluster_subscription, cluster_resource_group_name, cluster_name):
    from azure.cli.core.util import send_raw_request
    armendpoint = cmd.cli_ctx.cloud.endpoints.resource_manager
    feature_check_url = f"{armendpoint}/subscriptions/{cluster_subscription}/resourceGroups/{cluster_resource_group_name}/providers/Microsoft.ContainerService/managedClusters/{cluster_name}?api-version={AKS_CLUSTER_API}"
    try:
        headers = ['User-Agent=azuremonitormetrics.check_azuremonitormetrics_profile']
        r = send_raw_request(cmd.cli_ctx, "GET", feature_check_url,
                             body={}, headers=headers)
    except CLIError as e:
        raise UnknownError(e)
    json_response = json.loads(r.text)
    values_array = json_response["properties"]
    if "azureMonitorProfile" in values_array:
        if "metrics" in values_array["azureMonitorProfile"]:
            if values_array["azureMonitorProfile"]["metrics"]["enabled"] is True:
                return True
    return False



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import uuid
from knack.log import get_logger
from azure.mgmt.core.tools import parse_resource_id, is_valid_resource_id
from azure.cli.core.aaz import has_value
from .aaz.latest.managedservices.definition import (
    Create as _DefinitionCreate, Delete as _DefinitionDelete, List as _DefinitionList, Show as _DefinitionShow)
from .aaz.latest.managedservices.assignment import (
    Create as _AssignmentCreate, Delete as _AssignmentDelete, List as _AssignmentList, Show as _AssignmentShow)

logger = get_logger(__name__)

# region Definitions Custom Commands


# pylint: disable=unused-argument, protected-access
class DefinitionCreate(_DefinitionCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.auth_principal_id = AAZStrArg(
            options=["--principal-id"],
            help="The principal id.",
            required=True
        )
        args_schema.auth_role_definition_id = AAZStrArg(
            options=["--role-definition-id "],
            help="The role definition id.",
            required=True
        )
        args_schema.scope._registered = False
        args_schema.authorizations._registered = False
        args_schema.name._required = True
        args_schema.tenant_id._required = True
        args_schema.definition_id._required = False
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        from azure.cli.core.commands.client_factory import get_subscription_id
        args = self.ctx.args
        if not args.definition_id:
            args.definition_id = str(uuid.uuid4())

        subscription = get_subscription_id(self.cli_ctx)
        scope = _get_scope(subscription)
        args.scope = scope
        authorizations = {}
        if has_value(args.auth_principal_id):
            authorizations.update({
                'principalId': args.auth_principal_id.to_serialized_data()
            })
        if has_value(args.auth_role_definition_id):
            authorizations.update({
                'roleDefinitionId': args.auth_role_definition_id.to_serialized_data()
            })
        if authorizations:
            args.authorizations = [authorizations]
        if not has_value(args.plan_name) or not has_value(args.plan_product) \
                or not has_value(args.plan_publisher) or not has_value(args.plan_product):
            args.plan_name = None
            args.plan_product = None
            args.plan_publisher = None
            args.plan_version = None


class DefinitionShow(_DefinitionShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._registered = False
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        subscription = _get_subscription_id_from_cmd(self.cli_ctx)
        definition_id, sub_id, rg_name = _get_resource_id_parts(self.cli_ctx,
                                                                args.definition.to_serialized_data(), subscription)
        scope = _get_scope(sub_id, rg_name)
        args.scope = scope
        args.definition = definition_id


class DefinitionList(_DefinitionList):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._registered = False
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        from azure.cli.core.commands.client_factory import get_subscription_id
        args = self.ctx.args
        subscription = get_subscription_id(self.cli_ctx)
        scope = _get_scope(subscription)
        args.scope = scope


class DefinitionDelete(_DefinitionDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._registered = False
        args_schema.scope._required = False
        args_schema.definition._required = True
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        subscription = _get_subscription_id_from_cmd(self.cli_ctx)
        definition_id, sub_id, rg_name = _get_resource_id_parts(self.cli_ctx,
                                                                args.definition.to_serialized_data(), subscription)
        scope = _get_scope(sub_id, rg_name)
        args.scope = scope
        args.definition = definition_id


# endregion

# region Assignments Custom Commands

# pylint: disable=unused-argument

class AssignmentCreate(_AssignmentCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.resource_group_name = AAZStrArg(
            options=["--resource-group", "-g"],
            help="Name of resource group."
                 " You can configure the default group using `az configure --defaults group=<name>`",
        )
        args_schema.assignment_id._required = False
        args_schema.scope._registered = False
        args_schema.definition._required = True
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        resource_group_name = args.resource_group_name if has_value(args.resource_group_name) else None
        if not is_valid_resource_id(args.definition.to_serialized_data()):
            from azure.cli.core.azclierror import InvalidArgumentValueError
            raise InvalidArgumentValueError(
                "definition should be a valid resource id. For example, "
                "/subscriptions/id/providers/Microsoft.ManagedServices/registrationDefinitions/id")
        if not args.assignment_id:
            args.assignment_id = str(uuid.uuid4())

        subscription = _get_subscription_id_from_cmd(self.cli_ctx)
        scope = _get_scope(subscription, resource_group_name)
        args.scope = scope


class AssignmentShow(_AssignmentShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.resource_group_name = AAZStrArg(
            options=["--resource-group", "-g"],
            help="Name of resource group."
                 " You can configure the default group using `az configure --defaults group=<name>`",
        )
        args_schema.scope._registered = False
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        resource_group_name = args.resource_group_name if has_value(args.resource_group_name) else None
        subscription = _get_subscription_id_from_cmd(self.cli_ctx)
        assignment_id, sub_id, rg_name = _get_resource_id_parts(self.cli_ctx, args.assignment.to_serialized_data(),
                                                                subscription, resource_group_name)
        scope = _get_scope(sub_id, rg_name)
        args.scope = scope
        args.assignment = assignment_id


class AssignmentDelete(_AssignmentDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.resource_group_name = AAZStrArg(
            options=["--resource-group", "-g"],
            help="Name of resource group."
                 " You can configure the default group using `az configure --defaults group=<name>`",
        )
        args_schema.scope._registered = False
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        rg_name = args.resource_group_name.to_serialized_data() if has_value(args.resource_group_name) else None
        subscription = _get_subscription_id_from_cmd(self.cli_ctx)
        assignment_id, sub_id, rg_name = _get_resource_id_parts(self.cli_ctx, args.assignment.to_serialized_data(),
                                                                subscription, rg_name)
        scope = _get_scope(sub_id, rg_name)
        args.scope = scope
        args.assignment = assignment_id


class AssignmentList(_AssignmentList):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.resource_group_name = AAZStrArg(
            options=["--resource-group", "-g"],
            help="Name of resource group."
                 " You can configure the default group using `az configure --defaults group=<name>`",
        )
        args_schema.scope._registered = False
        args_schema.scope._required = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        resource_group_name = args.resource_group_name if has_value(args.resource_group_name) else None
        sub_id = _get_subscription_id_from_cmd(self.cli_ctx)
        scope = _get_scope(sub_id, resource_group_name)
        args.scope = scope


# endregion

# region private methods


def _get_subscription_id_from_cmd(cli_ctx):
    from azure.cli.core.commands.client_factory import get_subscription_id
    subscription = get_subscription_id(cli_ctx)
    return subscription


def _get_resource_id_parts(cli_ctx, name_or_id, subscription=None, resource_group_name=None):
    rg_name = None
    if is_valid_resource_id(name_or_id):
        id_parts = parse_resource_id(name_or_id)
        sub_id = id_parts['subscription']
        name = id_parts['name']
        if 'resource_group' in id_parts:
            rg_name = id_parts['resource_group']
    else:
        rg_name = resource_group_name
        name = name_or_id
        sub_id = _get_subscription_id_from_cmd(cli_ctx)
    return name, sub_id, rg_name


def _get_scope(subscription_id, resource_group_name=None):
    if not resource_group_name:
        scope = "/subscriptions/" + subscription_id
    else:
        scope = "/subscriptions/" + subscription_id + "/resourceGroups/" + resource_group_name

    return scope
# endregion



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Update(AAZCommand):
    """Update an existing schedule.
    """

    _aaz_info = {
        "version": "2018-09-15",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.devtestlab/schedules/{}", "2018-09-15"],
        ]
    }

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="The name of the schedule.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.daily_recurrence = AAZObjectArg(
            options=["--daily-recurrence"],
            arg_group="Properties",
            help="If the schedule will occur once each day of the week, specify the daily recurrence.",
            nullable=True,
        )
        _args_schema.hourly_recurrence = AAZObjectArg(
            options=["--hourly-recurrence"],
            arg_group="Properties",
            help="If the schedule will occur multiple times a day, specify the hourly recurrence.",
            nullable=True,
        )
        _args_schema.notification_settings = AAZObjectArg(
            options=["--notification-settings"],
            arg_group="Properties",
            help="Notification settings.",
            nullable=True,
        )
        _args_schema.status = AAZStrArg(
            options=["--status"],
            arg_group="Properties",
            help="The status of the schedule (i.e. Enabled, Disabled)",
            nullable=True,
            enum={"Disabled": "Disabled", "Enabled": "Enabled"},
        )
        _args_schema.target_resource_id = AAZStrArg(
            options=["--target-resource-id"],
            arg_group="Properties",
            help="The resource ID to which the schedule belongs",
            nullable=True,
        )
        _args_schema.task_type = AAZStrArg(
            options=["--task-type"],
            arg_group="Properties",
            help="The task type of the schedule (e.g. LabVmsShutdownTask, LabVmAutoStart).",
            nullable=True,
        )
        _args_schema.time_zone_id = AAZStrArg(
            options=["--time-zone-id"],
            arg_group="Properties",
            help="The time zone ID (e.g. China Standard Time, Greenland Standard Time, Pacific Standard time, etc.). The possible values for this property can be found in `IReadOnlyCollection<string> TimeZoneConverter.TZConvert.KnownWindowsTimeZoneIds` (https://github.com/mattjohnsonpint/TimeZoneConverter/blob/main/README.md)",
            nullable=True,
        )
        _args_schema.weekly_recurrence = AAZObjectArg(
            options=["--weekly-recurrence"],
            arg_group="Properties",
            help="If the schedule will occur only some days of the week, specify the weekly recurrence.",
            nullable=True,
        )

        daily_recurrence = cls._args_schema.daily_recurrence
        daily_recurrence.time = AAZStrArg(
            options=["time"],
            help="The time of day the schedule will occur.",
            nullable=True,
        )

        hourly_recurrence = cls._args_schema.hourly_recurrence
        hourly_recurrence.minute = AAZIntArg(
            options=["minute"],
            help="Minutes of the hour the schedule will run.",
            nullable=True,
        )

        notification_settings = cls._args_schema.notification_settings
        notification_settings.email_recipient = AAZStrArg(
            options=["email-recipient"],
            help="The email recipient to send notifications to (can be a list of semi-colon separated email addresses).",
            nullable=True,
        )
        notification_settings.notification_locale = AAZStrArg(
            options=["notification-locale"],
            help="The locale to use when sending a notification (fallback for unsupported languages is EN).",
            nullable=True,
        )
        notification_settings.status = AAZStrArg(
            options=["status"],
            help="If notifications are enabled for this schedule (i.e. Enabled, Disabled).",
            nullable=True,
            enum={"Disabled": "Disabled", "Enabled": "Enabled"},
        )
        notification_settings.time_in_minutes = AAZIntArg(
            options=["time-in-minutes"],
            help="Time in minutes before event at which notification will be sent.",
            nullable=True,
        )
        notification_settings.webhook_url = AAZStrArg(
            options=["webhook-url"],
            help="The webhook URL to which the notification will be sent.",
            nullable=True,
        )

        weekly_recurrence = cls._args_schema.weekly_recurrence
        weekly_recurrence.time = AAZStrArg(
            options=["time"],
            help="The time of the day the schedule will occur.",
            nullable=True,
        )
        weekly_recurrence.weekdays = AAZListArg(
            options=["weekdays"],
            help="The days of the week for which the schedule is set (e.g. Sunday, Monday, Tuesday, etc.).",
            nullable=True,
        )

        weekdays = cls._args_schema.weekly_recurrence.weekdays
        weekdays.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Schedule"

        _args_schema = cls._args_schema
        _args_schema.location = AAZResourceLocationArg(
            arg_group="Schedule",
            help="The location of the resource.",
            nullable=True,
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="Schedule",
            help="The tags of the resource.",
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.GlobalSchedulesGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        self.GlobalSchedulesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class GlobalSchedulesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DevTestLab/schedules/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2018-09-15",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_schedule_read(cls._schema_on_200)

            return cls._schema_on_200

    class GlobalSchedulesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DevTestLab/schedules/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2018-09-15",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_schedule_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("properties", AAZObjectType, ".", typ_kwargs={"flags": {"required": True, "client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("dailyRecurrence", AAZObjectType, ".daily_recurrence")
                properties.set_prop("hourlyRecurrence", AAZObjectType, ".hourly_recurrence")
                properties.set_prop("notificationSettings", AAZObjectType, ".notification_settings")
                properties.set_prop("status", AAZStrType, ".status")
                properties.set_prop("targetResourceId", AAZStrType, ".target_resource_id")
                properties.set_prop("taskType", AAZStrType, ".task_type")
                properties.set_prop("timeZoneId", AAZStrType, ".time_zone_id")
                properties.set_prop("weeklyRecurrence", AAZObjectType, ".weekly_recurrence")

            daily_recurrence = _builder.get(".properties.dailyRecurrence")
            if daily_recurrence is not None:
                daily_recurrence.set_prop("time", AAZStrType, ".time")

            hourly_recurrence = _builder.get(".properties.hourlyRecurrence")
            if hourly_recurrence is not None:
                hourly_recurrence.set_prop("minute", AAZIntType, ".minute")

            notification_settings = _builder.get(".properties.notificationSettings")
            if notification_settings is not None:
                notification_settings.set_prop("emailRecipient", AAZStrType, ".email_recipient")
                notification_settings.set_prop("notificationLocale", AAZStrType, ".notification_locale")
                notification_settings.set_prop("status", AAZStrType, ".status")
                notification_settings.set_prop("timeInMinutes", AAZIntType, ".time_in_minutes")
                notification_settings.set_prop("webhookUrl", AAZStrType, ".webhook_url")

            weekly_recurrence = _builder.get(".properties.weeklyRecurrence")
            if weekly_recurrence is not None:
                weekly_recurrence.set_prop("time", AAZStrType, ".time")
                weekly_recurrence.set_prop("weekdays", AAZListType, ".weekdays")

            weekdays = _builder.get(".properties.weeklyRecurrence.weekdays")
            if weekdays is not None:
                weekdays.set_elements(AAZStrType, ".")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    _schema_schedule_read = None

    @classmethod
    def _build_schema_schedule_read(cls, _schema):
        if cls._schema_schedule_read is not None:
            _schema.id = cls._schema_schedule_read.id
            _schema.location = cls._schema_schedule_read.location
            _schema.name = cls._schema_schedule_read.name
            _schema.properties = cls._schema_schedule_read.properties
            _schema.tags = cls._schema_schedule_read.tags
            _schema.type = cls._schema_schedule_read.type
            return

        cls._schema_schedule_read = _schema_schedule_read = AAZObjectType()

        schedule_read = _schema_schedule_read
        schedule_read.id = AAZStrType(
            flags={"read_only": True},
        )
        schedule_read.location = AAZStrType()
        schedule_read.name = AAZStrType(
            flags={"read_only": True},
        )
        schedule_read.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )
        schedule_read.tags = AAZDictType()
        schedule_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_schedule_read.properties
        properties.created_date = AAZStrType(
            serialized_name="createdDate",
            flags={"read_only": True},
        )
        properties.daily_recurrence = AAZObjectType(
            serialized_name="dailyRecurrence",
        )
        properties.hourly_recurrence = AAZObjectType(
            serialized_name="hourlyRecurrence",
        )
        properties.notification_settings = AAZObjectType(
            serialized_name="notificationSettings",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.status = AAZStrType()
        properties.target_resource_id = AAZStrType(
            serialized_name="targetResourceId",
        )
        properties.task_type = AAZStrType(
            serialized_name="taskType",
        )
        properties.time_zone_id = AAZStrType(
            serialized_name="timeZoneId",
        )
        properties.unique_identifier = AAZStrType(
            serialized_name="uniqueIdentifier",
            flags={"read_only": True},
        )
        properties.weekly_recurrence = AAZObjectType(
            serialized_name="weeklyRecurrence",
        )

        daily_recurrence = _schema_schedule_read.properties.daily_recurrence
        daily_recurrence.time = AAZStrType()

        hourly_recurrence = _schema_schedule_read.properties.hourly_recurrence
        hourly_recurrence.minute = AAZIntType()

        notification_settings = _schema_schedule_read.properties.notification_settings
        notification_settings.email_recipient = AAZStrType(
            serialized_name="emailRecipient",
        )
        notification_settings.notification_locale = AAZStrType(
            serialized_name="notificationLocale",
        )
        notification_settings.status = AAZStrType()
        notification_settings.time_in_minutes = AAZIntType(
            serialized_name="timeInMinutes",
        )
        notification_settings.webhook_url = AAZStrType(
            serialized_name="webhookUrl",
        )

        weekly_recurrence = _schema_schedule_read.properties.weekly_recurrence
        weekly_recurrence.time = AAZStrType()
        weekly_recurrence.weekdays = AAZListType()

        weekdays = _schema_schedule_read.properties.weekly_recurrence.weekdays
        weekdays.Element = AAZStrType()

        tags = _schema_schedule_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_schedule_read.id
        _schema.location = cls._schema_schedule_read.location
        _schema.name = cls._schema_schedule_read.name
        _schema.properties = cls._schema_schedule_read.properties
        _schema.tags = cls._schema_schedule_read.tags
        _schema.type = cls._schema_schedule_read.type


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import time
import unittest

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, live_only, StorageAccountPreparer,
                               LogAnalyticsWorkspacePreparer)

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
from .common import TEST_LOCATION
from .utils import create_containerapp_env
from azure.cli.command_modules.containerapp.tests.latest.common import (
    write_test_file,
    clean_up_test_file,
    TEST_DIR, TEST_LOCATION)

# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppMountAzureFileTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location=TEST_LOCATION)
    @LogAnalyticsWorkspacePreparer(location="eastus", get_shared_key=True)
    def test_container_app_mount_azurefile_e2e(self, resource_group, laworkspace_customer_id, laworkspace_shared_key):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        app = self.create_random_name(prefix='app1', length=24)
        storage = self.create_random_name(prefix='storage', length=24)
        share = self.create_random_name(prefix='share', length=10)

        self.cmd(
            f'az storage account create --resource-group {resource_group}  --name {storage} --location {TEST_LOCATION} --kind StorageV2 --sku Standard_LRS --enable-large-file-share --output none')
        self.cmd(
            f'az storage share-rm create --resource-group {resource_group}  --storage-account {storage} --name {share} --quota 1024 --enabled-protocols SMB --output none')

        create_containerapp_env(self, env, resource_group, logs_workspace=laworkspace_customer_id, logs_workspace_shared_key=laworkspace_shared_key)
        account_key = self.cmd(f'az storage account keys list -g {resource_group} -n {storage} --query "[0].value" '
                               '-otsv').output.strip()

        self.cmd(f'az containerapp env storage set -g {resource_group} -n {env} -a {storage} -k {account_key} -f {share} --storage-name {share} --access-mode ReadWrite')

        containerapp_env = self.cmd('containerapp env show -n {} -g {}'.format(env, resource_group), checks=[
            JMESPathCheck('name', env)

        ]).get_output_in_json()
        containerapp_yaml_text = f"""
                    location: {TEST_LOCATION}
                    type: Microsoft.App/containerApps
                    name: {app}
                    resourceGroup: {resource_group}
                    properties:
                        managedEnvironmentId: {containerapp_env["id"]}
                        configuration:
                            activeRevisionsMode: Single
                            ingress:
                                external: true
                                allowInsecure: true
                                targetPort: 80
                                traffic:
                                    - latestRevision: true
                                      weight: 100
                                transport: Auto
                        template:
                            containers:
                                - image: mcr.microsoft.com/k8se/quickstart:latest
                                  name: acamounttest
                                  resources:
                                      cpu: 0.5
                                      ephemeralStorage: 1Gi
                                      memory: 1Gi
                                  volumeMounts:
                                      - mountPath: /mnt/data
                                        volumeName: azure-files-volume
                                        subPath: sub
                            volumes:
                                - name: azure-files-volume
                                  storageType: AzureFile
                                  storageName: {share}
                                  mountOptions: uid=999,gid=999
                    """
        containerapp_file_name = f"{self._testMethodName}_containerapp.yml"

        write_test_file(containerapp_file_name, containerapp_yaml_text)
        self.cmd(
            f'az containerapp create -g {resource_group} --environment {env} -n {app} --yaml {containerapp_file_name}')

        self.cmd('containerapp show -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('properties.template.volumes[0].storageName', share),
            JMESPathCheck('properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('properties.template.volumes[0].mountOptions', 'uid=999,gid=999'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].subPath', 'sub'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume'),
        ])

        self.cmd('az containerapp revision list -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('[0].properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('[0].properties.template.volumes[0].storageName', share),
            JMESPathCheck('[0].properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('[0].properties.template.volumes[0].mountOptions', 'uid=999,gid=999'),
            JMESPathCheck('[0].properties.template.containers[0].volumeMounts[0].subPath', 'sub'),
            JMESPathCheck('[0].properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('[0].properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume'),
        ])
        clean_up_test_file(containerapp_file_name)
        containerapp_yaml_text = f"""
                           location: {TEST_LOCATION}
                           type: Microsoft.App/containerApps
                           name: {app}
                           resourceGroup: {resource_group}
                           properties:
                               managedEnvironmentId: {containerapp_env["id"]}
                               configuration:
                                   activeRevisionsMode: Single
                                   ingress:
                                       external: true
                                       allowInsecure: true
                                       targetPort: 80
                                       traffic:
                                           - latestRevision: true
                                             weight: 100
                                       transport: Auto
                               template:
                                   containers:
                                       - image: mcr.microsoft.com/k8se/quickstart:latest
                                         name: acamounttest
                                         resources:
                                             cpu: 0.5
                                             ephemeralStorage: 1Gi
                                             memory: 1Gi
                                         volumeMounts:
                                             - mountPath: /mnt/data
                                               volumeName: azure-files-volume
                                               subPath: sub2
                                   volumes:
                                       - name: azure-files-volume
                                         storageType: AzureFile
                                         storageName: {share}
                                         mountOptions: uid=1000,gid=1000
                           """
        containerapp_file_name = f"{self._testMethodName}_containerapp_1.yml"

        write_test_file(containerapp_file_name, containerapp_yaml_text)
        self.cmd(
            f'az containerapp update -g {resource_group} -n {app} --yaml {containerapp_file_name}')

        self.cmd('containerapp show -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('properties.template.volumes[0].storageName', share),
            JMESPathCheck('properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('properties.template.volumes[0].mountOptions', 'uid=1000,gid=1000'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].subPath', 'sub2'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume'),
        ])

        self.cmd('az containerapp revision list -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('[1].properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('[1].properties.template.volumes[0].storageName', share),
            JMESPathCheck('[1].properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('[1].properties.template.volumes[0].mountOptions', 'uid=1000,gid=1000'),
            JMESPathCheck('[1].properties.template.containers[0].volumeMounts[0].subPath', 'sub2'),
            JMESPathCheck('[1].properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('[1].properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume'),
        ])
        clean_up_test_file(containerapp_file_name)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "databoxedge device create",
)
class Create(AAZCommand):
    """Create a Data Box Edge/Data Box Gateway resource.

    :example: Create a Data Box Edge resource.
        az databoxedge device create --location "eastus" --sku "Edge" --name "testedgedevice" --resource-group "GroupForEdgeAutomation"
    """

    _aaz_info = {
        "version": "2021-02-01-preview",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.databoxedge/databoxedgedevices/{}", "2021-02-01-preview"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.device_name = AAZStrArg(
            options=["-n", "--name", "--device-name"],
            help="The device name.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.etag = AAZStrArg(
            options=["--etag"],
            help="The etag for the devices.",
        )
        _args_schema.location = AAZResourceLocationArg(
            help="The location of the device. This is a supported and registered Azure geographical region (for example, West US, East US, or Southeast Asia). The geographical region of a device cannot be changed once it is created, but if an identical geographical region is specified on update, the request will succeed.",
            required=True,
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.status = AAZStrArg(
            options=["--status"],
            help="The status of the Data Box Edge/Gateway device.",
            enum={"Disconnected": "Disconnected", "Maintenance": "Maintenance", "NeedsAttention": "NeedsAttention", "Offline": "Offline", "Online": "Online", "PartiallyDisconnected": "PartiallyDisconnected", "ReadyToSetup": "ReadyToSetup"},
        )
        _args_schema.description = AAZStrArg(
            options=["--description"],
            help="The Description of the Data Box Edge/Gateway device.",
        )
        _args_schema.friendly_name = AAZStrArg(
            options=["--friendly-name"],
            help="The Data Box Edge/Gateway device name.",
        )
        _args_schema.model_description = AAZStrArg(
            options=["--model-description"],
            help="The description of the Data Box Edge/Gateway device model.",
        )
        _args_schema.sku = AAZStrArg(
            options=["--sku"],
            help="SKU name.",
            enum={"EP2_128_1T4_Mx1_W": "EP2_128_1T4_Mx1_W", "EP2_256_2T4_W": "EP2_256_2T4_W", "EP2_64_1VPU_W": "EP2_64_1VPU_W", "Edge": "Edge", "EdgeMR_Mini": "EdgeMR_Mini", "EdgePR_Base": "EdgePR_Base", "EdgePR_Base_UPS": "EdgePR_Base_UPS", "EdgeP_Base": "EdgeP_Base", "EdgeP_High": "EdgeP_High", "GPU": "GPU", "Gateway": "Gateway", "Management": "Management", "RCA_Large": "RCA_Large", "RCA_Small": "RCA_Small", "RDC": "RDC", "TCA_Large": "TCA_Large", "TCA_Small": "TCA_Small", "TDC": "TDC", "TEA_1Node": "TEA_1Node", "TEA_1Node_Heater": "TEA_1Node_Heater", "TEA_1Node_UPS": "TEA_1Node_UPS", "TEA_1Node_UPS_Heater": "TEA_1Node_UPS_Heater", "TEA_4Node_Heater": "TEA_4Node_Heater", "TEA_4Node_UPS_Heater": "TEA_4Node_UPS_Heater", "TMA": "TMA"},
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="The list of tags that describe the device. These tags can be used to view and group this device (across resource groups).",
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.DevicesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class DevicesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBoxEdge/dataBoxEdgeDevices/{deviceName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "deviceName", self.ctx.args.device_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2021-02-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("etag", AAZStrType, ".etag")
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("sku", AAZObjectType)
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("dataBoxEdgeDeviceStatus", AAZStrType, ".status")
                properties.set_prop("description", AAZStrType, ".description")
                properties.set_prop("friendlyName", AAZStrType, ".friendly_name")
                properties.set_prop("modelDescription", AAZStrType, ".model_description")

            sku = _builder.get(".sku")
            if sku is not None:
                sku.set_prop("name", AAZStrType, ".sku")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType()
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.identity = AAZObjectType()
            _schema_on_200.kind = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            _schema_on_200.sku = AAZObjectType()
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            identity = cls._schema_on_200.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType()

            properties = cls._schema_on_200.properties
            properties.configured_role_types = AAZListType(
                serialized_name="configuredRoleTypes",
                flags={"read_only": True},
            )
            properties.culture = AAZStrType(
                flags={"read_only": True},
            )
            properties.data_box_edge_device_status = AAZStrType(
                serialized_name="dataBoxEdgeDeviceStatus",
            )
            properties.description = AAZStrType()
            properties.device_hcs_version = AAZStrType(
                serialized_name="deviceHcsVersion",
                flags={"read_only": True},
            )
            properties.device_local_capacity = AAZIntType(
                serialized_name="deviceLocalCapacity",
                flags={"read_only": True},
            )
            properties.device_model = AAZStrType(
                serialized_name="deviceModel",
                flags={"read_only": True},
            )
            properties.device_software_version = AAZStrType(
                serialized_name="deviceSoftwareVersion",
                flags={"read_only": True},
            )
            properties.device_type = AAZStrType(
                serialized_name="deviceType",
                flags={"read_only": True},
            )
            properties.edge_profile = AAZObjectType(
                serialized_name="edgeProfile",
                flags={"read_only": True},
            )
            properties.friendly_name = AAZStrType(
                serialized_name="friendlyName",
            )
            properties.model_description = AAZStrType(
                serialized_name="modelDescription",
            )
            properties.node_count = AAZIntType(
                serialized_name="nodeCount",
                flags={"read_only": True},
            )
            properties.resource_move_details = AAZObjectType(
                serialized_name="resourceMoveDetails",
                flags={"read_only": True},
            )
            properties.serial_number = AAZStrType(
                serialized_name="serialNumber",
                flags={"read_only": True},
            )
            properties.time_zone = AAZStrType(
                serialized_name="timeZone",
                flags={"read_only": True},
            )

            configured_role_types = cls._schema_on_200.properties.configured_role_types
            configured_role_types.Element = AAZStrType()

            edge_profile = cls._schema_on_200.properties.edge_profile
            edge_profile.subscription = AAZObjectType()

            subscription = cls._schema_on_200.properties.edge_profile.subscription
            subscription.id = AAZStrType()
            subscription.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            subscription.registration_date = AAZStrType(
                serialized_name="registrationDate",
            )
            subscription.registration_id = AAZStrType(
                serialized_name="registrationId",
            )
            subscription.state = AAZStrType()
            subscription.subscription_id = AAZStrType(
                serialized_name="subscriptionId",
            )

            properties = cls._schema_on_200.properties.edge_profile.subscription.properties
            properties.location_placement_id = AAZStrType(
                serialized_name="locationPlacementId",
            )
            properties.quota_id = AAZStrType(
                serialized_name="quotaId",
            )
            properties.registered_features = AAZListType(
                serialized_name="registeredFeatures",
            )
            properties.serialized_details = AAZStrType(
                serialized_name="serializedDetails",
            )
            properties.tenant_id = AAZStrType(
                serialized_name="tenantId",
            )

            registered_features = cls._schema_on_200.properties.edge_profile.subscription.properties.registered_features
            registered_features.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.edge_profile.subscription.properties.registered_features.Element
            _element.name = AAZStrType()
            _element.state = AAZStrType()

            resource_move_details = cls._schema_on_200.properties.resource_move_details
            resource_move_details.operation_in_progress = AAZStrType(
                serialized_name="operationInProgress",
            )
            resource_move_details.operation_in_progress_lock_timeout_in_utc = AAZStrType(
                serialized_name="operationInProgressLockTimeoutInUTC",
            )

            sku = cls._schema_on_200.sku
            sku.name = AAZStrType()
            sku.tier = AAZStrType()

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def load_arguments(self, _):  # pylint: disable=unused-argument
    with self.argument_context('lab vm claim') as c:
        c.argument('name', options_list=['--name', '-n'], id_part='child_name_1',
                   help='Name of the virtual machine to claim.')
        c.argument('lab_name', id_part='name', help='Name of the lab.')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
import os

from azure.cli.telemetry.util import save_payload

__version__ = "1.1.0"

DEFAULT_INSTRUMENTATION_KEY = 'c4395b75-49cc-422c-bc95-c7d51aef5d46'


def _start(config_dir, cache_dir):
    import subprocess
    from azure.cli.telemetry.components.telemetry_logging import get_logger

    logger = get_logger('process')

    args = [sys.executable, os.path.realpath(__file__), config_dir, cache_dir]
    logger.info('Creating upload process: "%s %s %s %s"', *args)

    kwargs = {'args': args}
    if os.name == 'nt':
        # Windows process creation flag to not reuse the parent console.
        # Without this, the background service is associated with the
        # starting process's console, and will block that console from
        # exiting until the background service self-terminates.
        # Elsewhere, fork just does the right thing.
        kwargs['creationflags'] = 0x00000010  # CREATE_NEW_CONSOLE

        startupinfo = subprocess.STARTUPINFO()
        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        startupinfo.wShowWindow = subprocess.SW_HIDE
        kwargs['startupinfo'] = startupinfo
    else:
        kwargs['stdin'] = subprocess.DEVNULL
        kwargs['stdout'] = subprocess.DEVNULL
        kwargs['stderr'] = subprocess.STDOUT

    process = subprocess.Popen(**kwargs)
    logger.info('Return from creating process %s', process.pid)


def save(config_dir, payload):
    from azure.cli.telemetry.components.telemetry_logging import get_logger

    logger = get_logger('main')
    try:
        # Split payload to cli events and extra events by instrumentation key
        # extra events can be sent immediately
        # cli events will be handled in separate process
        import json
        events = json.loads(payload)

        logger.info('Begin splitting cli events and extra events, total events: %s', len(events))
        cli_events = {}
        extra_events = {}
        for key, event in events.items():
            if key == DEFAULT_INSTRUMENTATION_KEY:
                cli_events[key] = event
            else:
                extra_events[key] = event

        if extra_events:
            # Defer applicationinsights import to only when extra events exist
            from azure.cli.telemetry.components.telemetry_client import CliTelemetryClient
            client = CliTelemetryClient()
            for key, event in extra_events.items():
                client.add(json.dumps({key: event}), flush=True)
            client.flush(force=True)

        cli_payload = json.dumps(cli_events) if cli_events else None
        logger.info('Finish splitting cli events and extra events, cli events: %s', len(cli_events))
    except Exception as ex:  # pylint: disable=broad-except
        logger.info("Split cli events and extra events failure: %s", str(ex))
        cli_payload = payload

    cache_dir = save_payload(config_dir, cli_payload)
    if cache_dir:
        logger.info('Begin creating telemetry upload process.')
        _start(config_dir, cache_dir)
        logger.info('Finish creating telemetry upload process.')


def main():
    from azure.cli.telemetry.components.records_collection import RecordsCollection
    from azure.cli.telemetry.components.telemetry_client import CliTelemetryClient
    from azure.cli.telemetry.components.telemetry_logging import config_logging_for_upload, get_logger

    try:
        config_dir = sys.argv[1]
        cache_dir = sys.argv[2]
        config_logging_for_upload(config_dir)

        logger = get_logger('main')
        logger.info('Attempt start. Configuration directory [%s]. Cache directory [%s].', sys.argv[1], sys.argv[2])

        try:
            import portalocker
            collection = RecordsCollection(cache_dir)
            collection.snapshot_and_read()

            client = CliTelemetryClient()
            for each in collection:
                client.add(each, flush=True)
            client.flush(force=True)
        except portalocker.AlreadyLocked:
            # another upload process is running.
            logger.info('Lock out from note file under %s which means another process is running. Exit 0.', config_dir)
            sys.exit(0)
        except OSError as err:
            logger.warning('Unexpected IO Error %s. Exit 1.', err)
            sys.exit(1)
        except Exception as err:  # pylint: disable=broad-except
            logger.error('Unexpected Error %s. Exit 2.', err)
            logger.exception(err)
            sys.exit(2)
    except IndexError:
        sys.exit(1)


if __name__ == '__main__':
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import time
import yaml

from azure.cli.core.azclierror import ValidationError
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, live_only, JMESPathCheckNotExists,
                               JMESPathCheckExists)

from azure.cli.command_modules.containerapp.tests.latest.common import (write_test_file, clean_up_test_file)
from .common import TEST_LOCATION

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppWorkloadProfilesTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    def test_containerapp_env_workload_profiles_list_supported(self):
        self.cmd('containerapp env workload-profile list-supported -l northeurope', checks=[
            JMESPathCheck('[?name==`NC48-A100`].properties.gpus', [2]),
            JMESPathCheck('[?name==`NC96-A100`].properties.gpus', [4]),
            JMESPathCheckExists('[?name==`NC48-A100`]'),
            JMESPathCheckExists('[?name==`NC96-A100`]'),
            JMESPathCheckExists('[?name==`D4`]'),
            JMESPathCheckExists('[?name==`D8`]'),
            JMESPathCheckExists('[?name==`D16`]'),
            JMESPathCheckExists('[?name==`D32`]'),
            JMESPathCheckExists('[?name==`E4`]'),
            JMESPathCheckExists('[?name==`E8`]'),
            JMESPathCheckExists('[?name==`E16`]'),
            JMESPathCheckExists('[?name==`E32`]'),
            JMESPathCheckExists('[?name==`Consumption`]'),
            JMESPathCheck('[?name==`D4`].properties.category', ["GeneralPurpose"]),
            JMESPathCheck('[?name==`D4`].properties.cores', [4]),
            JMESPathCheck('[?name==`D4`].properties.displayName', ["Dedicated-D4"]),
            JMESPathCheck('[?name==`D4`].properties.memoryGiB', [16]),
        ])

        self.cmd('containerapp env workload-profile list-supported -l eastus', checks=[
            JMESPathCheckNotExists('[?name==`NC48-A100`]'),
            JMESPathCheckNotExists('[?name==`NC96-A100`]'),
            JMESPathCheckExists('[?name==`D4`]'),
            JMESPathCheckExists('[?name==`D8`]'),
            JMESPathCheckExists('[?name==`D16`]'),
            JMESPathCheckExists('[?name==`D32`]'),
            JMESPathCheckExists('[?name==`E4`]'),
            JMESPathCheckExists('[?name==`E8`]'),
            JMESPathCheckExists('[?name==`E16`]'),
            JMESPathCheckExists('[?name==`E32`]'),
            JMESPathCheckExists('[?name==`Consumption`]'),
            JMESPathCheck('[?name==`D4`].properties.category', ["GeneralPurpose"]),
            JMESPathCheck('[?name==`D4`].properties.cores', [4]),
            JMESPathCheck('[?name==`D4`].properties.displayName', ["Dedicated-D4"]),
            JMESPathCheck('[?name==`D4`].properties.memoryGiB', [16]),
        ])

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    @live_only()  # encounters 'CannotOverwriteExistingCassetteException' only when run from recording (passes when run live)
    def test_containerapp_env_workload_profiles_e2e(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        vnet = self.create_random_name(prefix='name', length=24)
        app1 = self.create_random_name(prefix='app1', length=24)
        app2 = self.create_random_name(prefix='app2', length=24)

        self.cmd('containerapp env create -g {} -n {} --location {}  --logs-destination none --enable-workload-profiles'.format(resource_group, env, TEST_LOCATION))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(60)

        self.cmd('containerapp env show -n {} -g {}'.format(env, resource_group), checks=[
            JMESPathCheck('name', env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd("az containerapp env workload-profile list-supported -l {}".format(TEST_LOCATION))

        profiles = self.cmd("az containerapp env workload-profile list -g {} -n {}".format(resource_group, env)).get_output_in_json()
        self.assertEqual(len(profiles), 1)
        self.assertEqual(profiles[0]["properties"]["name"].lower(), "consumption")
        self.assertEqual(profiles[0]["properties"]["workloadProfileType"].lower(), "consumption")

        self.cmd("az containerapp env workload-profile add -g {} -n {} --workload-profile-name my-d4 --workload-profile-type D4 --min-nodes 2 --max-nodes 3".format(resource_group, env))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(60)
        self.cmd('containerapp env show -n {} -g {}'.format(env, resource_group), checks=[
            JMESPathCheck('name', env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd("az containerapp env workload-profile show -g {} -n {} --workload-profile-name my-d4 ".format(resource_group, env), checks=[
            JMESPathCheck("properties.name", "my-d4"),
            JMESPathCheck("properties.maximumCount", 3),
            JMESPathCheck("properties.minimumCount", 2),
            JMESPathCheck("properties.workloadProfileType", "D4"),
        ])

        self.cmd("az containerapp env workload-profile update -g {} -n {} --workload-profile-name my-d4 --min-nodes 1 --max-nodes 2".format(resource_group, env))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(60)
        self.cmd('containerapp env show -n {} -g {}'.format(env, resource_group), checks=[
            JMESPathCheck('name', env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd("az containerapp env workload-profile show -g {} -n {} --workload-profile-name my-d4 ".format(resource_group, env), checks=[
            JMESPathCheck("properties.name", "my-d4"),
            JMESPathCheck("properties.maximumCount", 2),
            JMESPathCheck("properties.minimumCount", 1),
            JMESPathCheck("properties.workloadProfileType", "D4"),
        ])

        self.cmd("az containerapp create -g {} --target-port 80 --ingress external --image mcr.microsoft.com/k8se/quickstart:latest --environment {} -n {} --workload-profile-name Consumption".format(resource_group, env, app1))
        self.cmd("az containerapp create -g {} --target-port 80 --ingress external --image mcr.microsoft.com/k8se/quickstart:latest --environment {} -n {} --workload-profile-name my-d4".format(resource_group, env, app2))
        self.cmd('containerapp delete -g {} -n {} --yes'.format(resource_group, app1), expect_failure=False)
        self.cmd('containerapp delete -g {} -n {} --yes'.format(resource_group, app2), expect_failure=False)
        self.cmd('containerapp env delete -g {} -n {} --yes'.format(resource_group, env), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    def test_containerapp_create_enable_workload_profiles_three_state_flag(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none --enable-workload-profiles false'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.workloadProfiles", None),
        ])

        with self.assertRaisesRegex(ValidationError,
                                     "Cannot add workload profile because the environment doesn't enable workload profile.\n"
                                     "If you want to use Consumption and Dedicated environment, please create a new one with 'az containerapp env create'."):
            self.cmd(
                'containerapp env workload-profile add -w wp -g {} -n {} --workload-profile-type D4 --min-nodes 1 --max-nodes 2'.format(
                    resource_group, env))

        with self.assertRaisesRegex(ValidationError,
                                     "Cannot update workload profile because the environment doesn't enable workload profile.\n"
                                     "If you want to use Consumption and Dedicated environment, please create a new one with 'az containerapp env create'."):
            self.cmd(
                'containerapp env workload-profile update -w wp -g {} -n {} --min-nodes 1 --max-nodes 2'.format(
                    resource_group, env))

        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env), expect_failure=False)

        env1 = self.create_random_name(prefix='env1', length=24)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none'.format(
            resource_group, env1), expect_failure=False, checks=[
            JMESPathCheck("name", env1),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])
        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env1), expect_failure=False)

        env2 = self.create_random_name(prefix='env2', length=24)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none --enable-workload-profiles'.format(
            resource_group, env2), expect_failure=False, checks=[
            JMESPathCheck("name", env2),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])
        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env2),
                 expect_failure=False)

        env3 = self.create_random_name(prefix='env3', length=24)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none --enable-workload-profiles true'.format(
            resource_group, env3), expect_failure=False, checks=[
            JMESPathCheck("name", env3),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])
        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env3), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    @live_only()  # encounters 'CannotOverwriteExistingCassetteException' only when run from recording (passes when run live)
    def test_containerapp_env_workload_profiles_delete(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        vnet = self.create_random_name(prefix='name', length=24)

        self.cmd("az network vnet create -l {} --address-prefixes '14.0.0.0/16' -g {} -n {}".format(TEST_LOCATION, resource_group, vnet))
        sub_id = self.cmd("az network vnet subnet create --address-prefixes '14.0.0.0/22' --delegations Microsoft.App/environments -n sub -g {} --vnet-name {}".format(resource_group, vnet)).get_output_in_json()["id"]

        self.cmd('containerapp env create -g {} -n {} -s {} --location {}  --logs-destination none --enable-workload-profiles'.format(resource_group, env, sub_id, TEST_LOCATION))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        self.cmd("az containerapp env workload-profile add -g {} -n {} --workload-profile-name my-d8 --workload-profile-type D8 --min-nodes 1 --max-nodes 1".format(resource_group, env))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        self.cmd("az containerapp env workload-profile show -g {} -n {} --workload-profile-name my-d8 ".format(resource_group, env), checks=[
            JMESPathCheck("properties.name", "my-d8"),
            JMESPathCheck("properties.maximumCount", 1),
            JMESPathCheck("properties.minimumCount", 1),
            JMESPathCheck("properties.workloadProfileType", "D8"),
        ])

        profiles = self.cmd("az containerapp env workload-profile list -g {} -n {}".format(resource_group, env)).get_output_in_json()
        self.assertEqual(len(profiles), 2)

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        self.cmd("az containerapp env workload-profile delete -g {} -n {} --workload-profile-name my-d8 ".format(resource_group, env))

        profiles = self.cmd("az containerapp env workload-profile list -g {} -n {}".format(resource_group, env)).get_output_in_json()
        self.assertEqual(len(profiles), 1)
        self.cmd('containerapp env delete -g {} -n {} --yes'.format(resource_group, env), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    def test_containerapp_create_with_workloadprofile_yaml(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        env = self.create_random_name(prefix='env', length=24)
        app = self.create_random_name(prefix='yaml', length=24)

        containerapp_env = self.cmd('containerapp env create -g {} -n {} --location {}  --logs-destination none --enable-workload-profiles'.format(resource_group, env, TEST_LOCATION)).get_output_in_json()
        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(120)

        self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env), checks=[
            JMESPathCheck('name', env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ]).get_output_in_json()

        workload_profile_name = "my-e16"

        self.cmd("az containerapp env workload-profile add -g {} -n {} --workload-profile-name {} --workload-profile-type E16 --min-nodes 1 --max-nodes 3".format(resource_group, env, workload_profile_name))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env), checks=[
            JMESPathCheck('name', env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ]).get_output_in_json()

        self.cmd("az containerapp env workload-profile show -g {} -n {} --workload-profile-name my-e16 ".format(resource_group, env), checks=[
            JMESPathCheck("properties.name", workload_profile_name),
            JMESPathCheck("properties.maximumCount", 3),
            JMESPathCheck("properties.minimumCount", 1),
            JMESPathCheck("properties.workloadProfileType", "E16"),
        ])

        revision_01 = "revision01"
        containerapp_yaml_text_01 = f"""
            location: {TEST_LOCATION}
            type: Microsoft.App/containerApps
            tags:
                tagname: value
            properties:
              managedEnvironmentId: {containerapp_env["id"]}
              workloadProfileName: {workload_profile_name}
              configuration:
                activeRevisionsMode: Multiple
                ingress:
                  external: true
                  allowInsecure: false
                  targetPort: 80
                  traffic:
                    - latestRevision: true
                      weight: 100
                  transport: Auto
                  ipSecurityRestrictions:
                    - name: name
                      ipAddressRange: "1.1.1.1/10"
                      action: "Allow"
              template:
                revisionSuffix: {revision_01}
                containers:
                  - image: nginx
                    name: nginx
                    env:
                      - name: HTTP_PORT
                        value: 80
                    command:
                      - npm
                      - start
                    resources:
                      cpu: 10.0
                      memory: 5Gi
                scale:
                  minReplicas: 1
                  maxReplicas: 3
            """
        containerapp_file_name_01 = f"{self._testMethodName}_containerapp_01.yml"

        write_test_file(containerapp_file_name_01, containerapp_yaml_text_01)
        self.cmd(f'containerapp create -n {app} -g {resource_group} --environment {env} --yaml {containerapp_file_name_01}')

        self.assertContainerappProperties(containerapp_env, resource_group, app, workload_profile_name, revision_01, "10.0", "5Gi")

        revision_02 = "revision02"
        containerapp_yaml_text_02 = containerapp_yaml_text_01.replace(workload_profile_name, "Consumption")
        containerapp_yaml_text_02 = containerapp_yaml_text_02.replace("cpu: 10.0", "cpu: 0.5")
        containerapp_yaml_text_02 = containerapp_yaml_text_02.replace("memory: 5Gi", "memory: 1Gi")
        containerapp_yaml_text_02 = containerapp_yaml_text_02.replace(revision_01, revision_02)

        containerapp_file_name_02 = f"{self._testMethodName}_containerapp_02.yml"
        write_test_file(containerapp_file_name_02, containerapp_yaml_text_02)

        self.cmd(f'containerapp update -n {app} -g {resource_group} --yaml {containerapp_file_name_02}')

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        self.assertContainerappProperties(containerapp_env, resource_group, app, "Consumption", revision_02, "0.5", "1Gi")

        clean_up_test_file(containerapp_file_name_01)
        clean_up_test_file(containerapp_file_name_02)
        self.cmd('containerapp delete -g {} -n {} --yes'.format(resource_group, app), expect_failure=False)
        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    def test_containerapp_env_workload_profiles_e2e_no_waits(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        vnet = self.create_random_name(prefix='name', length=24)
        app1 = self.create_random_name(prefix='app1', length=24)
        app2 = self.create_random_name(prefix='app2', length=24)

        self.cmd('containerapp env create -g {} -n {} --location {}  --logs-destination none --enable-workload-profiles'.format(resource_group, env, TEST_LOCATION))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        self.cmd('containerapp env show -n {} -g {}'.format(env, resource_group), checks=[
            JMESPathCheck('name', env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd("az containerapp env workload-profile list-supported -l {}".format(TEST_LOCATION))

        profiles = self.cmd("az containerapp env workload-profile list -g {} -n {}".format(resource_group, env)).get_output_in_json()
        self.assertEqual(len(profiles), 1)
        self.assertEqual(profiles[0]["properties"]["name"].lower(), "consumption")
        self.assertEqual(profiles[0]["properties"]["workloadProfileType"].lower(), "consumption")

        self.cmd("az containerapp env workload-profile add -g {} -n {} --workload-profile-name my-d4 --workload-profile-type D4 --min-nodes 2 --max-nodes 3".format(resource_group, env))

        self.cmd("az containerapp create -g {} --target-port 80 --ingress external --image mcr.microsoft.com/k8se/quickstart:latest --revision-suffix suf1 --environment {} -n {} --workload-profile-name my-d4 --cpu 0.5 --memory 1Gi".format(resource_group, env, app1))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        self.cmd(f'containerapp show -g {resource_group} -n {app1}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.template.revisionSuffix", "suf1"),
            JMESPathCheck("properties.workloadProfileName", "my-d4"),
            JMESPathCheck("properties.template.containers[0].resources.cpu", "0.5"),
            JMESPathCheck("properties.template.containers[0].resources.memory", "1Gi")
        ])

        self.cmd("az containerapp env workload-profile update -g {} -n {} --workload-profile-name my-d4 --min-nodes 1 --max-nodes 2".format(resource_group, env))

        self.cmd("az containerapp create -g {} --target-port 80 --ingress external --image mcr.microsoft.com/k8se/quickstart:latest --revision-suffix suf2 --environment {} -n {} --workload-profile-name my-d4 --cpu 0.5 --memory 1Gi".format(resource_group, env, app2))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        self.cmd(f'containerapp show -g {resource_group} -n {app2}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.template.revisionSuffix", "suf2"),
            JMESPathCheck("properties.workloadProfileName", "my-d4"),
            JMESPathCheck("properties.template.containers[0].resources.cpu", "0.5"),
            JMESPathCheck("properties.template.containers[0].resources.memory", "1Gi")
        ])
        self.cmd('containerapp delete -g {} -n {} --yes'.format(resource_group, app1), expect_failure=False)
        self.cmd('containerapp delete -g {} -n {} --yes'.format(resource_group, app2), expect_failure=False)
        self.cmd('containerapp env delete -g {} -n {} --yes'.format(resource_group, env), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerapp_env_workload_profiles_name_validation(self, resource_group):
        # Set location to a region that supports flex workload profiles
        TEST_LOCATION = "northcentralusstage"
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)

        self.cmd('containerapp env create -g {} -n {} --location {} --logs-destination none --enable-workload-profiles'.format(resource_group, env, TEST_LOCATION))

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        # Test 1: Creating a flex workload-profile should succeed without providing the workload-profile-name
        self.cmd('containerapp env workload-profile add -g {} -n {} --workload-profile-type flex'.format(resource_group, env), expect_failure=False)

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        # Verify the flex profile was created with a default name
        profiles = self.cmd("az containerapp env workload-profile list -g {} -n {}".format(resource_group, env)).get_output_in_json()
        flex_profiles = [p for p in profiles if p["properties"]["workloadProfileType"].lower() == "flex"]
        self.assertEqual(len(flex_profiles), 1)

        # Test 2: Creating a D4 workload-profile without a name should succeed and use "d4" as the default name
        self.cmd('containerapp env workload-profile add -g {} -n {} --workload-profile-type D4 --min-nodes 1 --max-nodes 2'.format(resource_group, env), expect_failure=False)

        containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        while containerapp_env["properties"]["provisioningState"].lower() in ["waiting", "inprogress"]:
            time.sleep(5)
            containerapp_env = self.cmd('containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()
        time.sleep(30)

        # Verify the D4 profile was created with a default name "d4"
        profiles = self.cmd("az containerapp env workload-profile list -g {} -n {}".format(resource_group, env)).get_output_in_json()
        d4_profiles = [p for p in profiles if p["properties"]["workloadProfileType"].lower() == "d4"]
        self.assertEqual(len(d4_profiles), 1)
        self.assertEqual(d4_profiles[0]["properties"]["name"].lower(), "d4")

        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    def test_containerapp_env_enable_workload_profiles_infer_env_type_v1(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        env = self.create_random_name(prefix='env', length=24)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none --enable-workload-profiles false'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.workloadProfiles", None),
        ])

        self.cmd('containerapp env create -g {} -n {} --enable-workload-profiles --logs-destination none'.format(resource_group, env), expect_failure=True)
        self.cmd('containerapp env create -g {} -n {} -w --logs-destination none'.format(resource_group, env), expect_failure=True)

        self.cmd('containerapp env create -g {} -n {} -w false --logs-destination none'.format(resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.workloadProfiles", None),
        ])

        self.cmd('containerapp env create -g {} -n {} --enable-workload-profiles false --logs-destination none'.format(resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.workloadProfiles", None),
        ])

        self.cmd('containerapp env create -g {} -n {} --logs-destination none'.format(resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.workloadProfiles", None),
        ])

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    def test_containerapp_env_enable_workload_profiles_infer_env_type_v2(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        env = self.create_random_name(prefix='env1', length=24)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd('containerapp env create -g {} -n {} --logs-destination none'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd('containerapp env create -g {} -n {} --enable-workload-profiles --logs-destination none'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd('containerapp env create -g {} -n {} -w --logs-destination none'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("length(properties.workloadProfiles)", 1),
            JMESPathCheck('properties.workloadProfiles[0].name', "Consumption", case_sensitive=False),
            JMESPathCheck('properties.workloadProfiles[0].workloadProfileType', "Consumption", case_sensitive=False),
        ])

        self.cmd('containerapp env create -g {} -n {} --enable-workload-profiles false --logs-destination none'.format(
            resource_group, env), expect_failure=True)

        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env), expect_failure=False)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="eastus")
    def test_containerapp_env_update_workload_profile_on_consumption_only_env(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        # Create environment with workload profiles disabled (consumption-only)
        self.cmd('containerapp env create -g {} -n {} --logs-destination none --enable-workload-profiles false'.format(
            resource_group, env), expect_failure=False, checks=[
            JMESPathCheck("name", env),
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.workloadProfiles", None),
        ])

        # Attempt to update environment with workload profile parameters should fail
        with self.assertRaisesRegex(ValidationError,
                                    "This environment does not allow for workload profiles. You can create a compatible environment with 'az containerapp env create --enable-workload-profiles'"):
            self.cmd('containerapp env update -g {} -n {} --workload-profile-name my-d4 --workload-profile-type D4 --min-nodes 1 --max-nodes 2'.format(
                resource_group, env))

        self.cmd('containerapp env delete -g {} -n {} --yes --no-wait'.format(resource_group, env), expect_failure=False)

    def assertContainerappProperties(self, containerapp_env, rg, app, workload_profile_name, revision, cpu, mem):
        self.cmd(f'containerapp show -g {rg} -n {app}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.configuration.ingress.external", True),
            JMESPathCheck("properties.configuration.ingress.ipSecurityRestrictions[0].name", "name"),
            JMESPathCheck("properties.configuration.ingress.ipSecurityRestrictions[0].ipAddressRange", "1.1.1.1/10"),
            JMESPathCheck("properties.configuration.ingress.ipSecurityRestrictions[0].action", "Allow"),
            JMESPathCheck("properties.environmentId", containerapp_env["id"]),
            JMESPathCheck("properties.template.revisionSuffix", revision),
            JMESPathCheck("properties.template.containers[0].name", "nginx"),
            JMESPathCheck("properties.template.scale.minReplicas", 1),
            JMESPathCheck("properties.template.scale.maxReplicas", 3),
            JMESPathCheck("properties.workloadProfileName", workload_profile_name),
            JMESPathCheck("properties.template.containers[0].resources.cpu", cpu),
            JMESPathCheck("properties.template.containers[0].resources.memory", mem)
        ])